
import datetime
import pytest

from django.utils.timezone import make_naive
from django.core.management import (
    call_command,
    find_commands,
    load_command_class,
)
from django.test import TestCase
import django_peeringdb.models
from django_peeringdb import settings

# set this to 0 to fetch all objects during sync test (takes forever)
# setting this to anything > 0 will trigger missing object validation errors
# causing the test to also test the fallback solution for that scenario
FETCH_LIMIT = 3


def sync_test(f):
    """
    check for sync settings, configure
    return decorator for all tests that sync
    """
    sync = pytest.config.getoption("--sync")

    # check explicitly for False, none means default (which is prod)
    if sync == False:
        return pytest.mark.skip(reason="need --sync option to run sync tests")(f)

    f.sync_args = dict(
        debug=pytest.config.getoption('--sync-debug'),
        )

    sync_only = pytest.config.getoption("--sync-only")
    if sync_only:
        f.sync_args['only'] = sync_only

    sync_id = pytest.config.getoption("--sync-id")
    if sync_id:
        f.sync_args['id'] = sync_id

    if not sync or sync == 'prod':
        pass

    elif sync == 'beta':
        settings.SYNC_URL = 'https://beta.peeringdb.com/api'

    else:
        settings.SYNC_URL = sync


    return f


@sync_test
class SyncTests(TestCase):
    """ test sync command """

    def setUp(self):
        self.cmd = load_command_class('django_peeringdb', 'pdb_sync')

    def test_get_since_empty(self):
        for cls in self.cmd.get_class_list():
            if cls.handleref.tag == "poc":
                continue
            assert 0 == self.cmd.get_since(cls)

    def test_sync_all(self):
        kwargs = getattr(self, 'sync_args', {})
        if "limit" not in kwargs:
            kwargs["limit"] = FETCH_LIMIT
        print("syncing kwargs {}".format(kwargs))
        self.cmd.handle(**kwargs)
#        self.cmd.sync(self.cmd.get_class_list())

#        for cls in self.cmd.get_class_list():
#            data = self.cmd.get_objs(cls)
#            for row in data:
#                for field in cls._meta.get_fields():
#                    value = cls._meta.get_field(field.name)
#                    if isinstance(value, datetime.datetime):
#                        #cls._meta.get_field(field.name).make_naive()
#                        setattr(cls, field.name, make_naive(value))
#
#            self.cmd.update_db(cls, data)
#
#            self.update_db(cls, self.get_objs(cls))

        for cls in self.cmd.get_class_list():
            if cls.handleref.tag == "poc":
                continue
            assert 0 != self.cmd.get_since(cls)
            assert 0 != cls.objects.all().count()
            print(cls.objects.all().count())

    def test_sync_call_command(self):
        call_command('pdb_sync', interactive=False, limit=FETCH_LIMIT)




from django.conf import settings


# lazy init for translations
_ = lambda s: s


def pytest_addoption(parser):
    parser.addoption("--sync", nargs='?', default=False,
                     help="run sync tests (optionally can set the target to either prod, beta, or a URL)")
    parser.addoption("--sync-debug", action='store_true',
                     help="enable sync debug")
    parser.addoption("--sync-only", nargs='+',
                     help="sync only these comma separated tables")
    parser.addoption("--sync-id", help="sync only this id")


def pytest_configure():

    settings.configure(
        INSTALLED_APPS=[
            'django.contrib.auth',
            'django.contrib.contenttypes',
            'django.contrib.admin',
            'django.contrib.sessions',
            'django_peeringdb',
        ],
        DATABASE_ENGINE='django.db.backends.sqlite3',
        DATABASES={
            'default': {
                'ENGINE': 'django.db.backends.sqlite3',
                'NAME': ':memory:',
            }
        },
        DEBUG=False,
        TEMPLATE_DEBUG=True,
        LOGGING = {
            'version': 1,
            'disable_existing_loggers': False,
            'handlers': {
                'stderr': {
                    'level': 'DEBUG',
                    'class': 'logging.StreamHandler',
                    },
            },
            'loggers': {
                '': {
                    'handlers': ['stderr'],
                    'level': 'DEBUG',
                    'propagate': False
                },
            },
        },

        USE_TZ=False,
        PEERINGDB_SYNC_STRIP_TZ=True,
        # add user defined iso code for Kosovo
        COUNTRIES_OVERRIDE = {
            'XK': _('Kosovo'),
        }
    )




MEDIA = (('Ethernet', 'Ethernet'),
         ('ATM', 'ATM'),
         ('Multiple', 'Multiple'))

POC_ROLES = (
    ('Abuse', 'Abuse'),
    ('Maintenance', 'Maintenance'),
    ('Policy', 'Policy'),
    ('Technical', 'Technical'),
    ('NOC', 'NOC'),
    ('Public Relations', 'Public Relations'),
    ('Sales', 'Sales')
)


POLICY_GENERAL = (('Open', 'Open'),
                  ('Selective', 'Selective'),
                  ('Restrictive', 'Restrictive'),
                  ('No', 'No'))
POLICY_LOCATIONS = (('Not Required', 'Not Required'),
                    ('Preferred', 'Preferred'),
                    ('Required - US', 'Required - US'),
                    ('Required - EU', 'Required - EU'),
                    ('Required - International',
                     'Required - International'))
POLICY_CONTRACTS = (('Not Required', 'Not Required'),
                    ('Private Only', 'Private Only'),
                    ('Required', 'Required'))

PROTOCOLS = (
    ('IPv4', 'IPv4'),
    ('IPv6', 'IPv6'),
)

RATIOS = (
    ('', 'Not Disclosed'),
    ('Not Disclosed', 'Not Disclosed'),
    ('Heavy Outbound', 'Heavy Outbound'),
    ('Mostly Outbound', 'Mostly Outbound'),
    ('Balanced', 'Balanced'),
    ('Mostly Inbound', 'Mostly Inbound'),
    ('Heavy Inbound', 'Heavy Inbound')
)

REGIONS = (('North America', 'North America'),
           ('Asia Pacific', 'Asia Pacific'),
           ('Europe', 'Europe'),
           ('South America', 'South America'),
           ('Africa', 'Africa'),
           ('Australia', 'Australia'),
           ('Middle East', 'Middle East'))

SCOPES = (
    ('', 'Not Disclosed'),
    ('Not Disclosed', 'Not Disclosed'),
    ('Regional', 'Regional'),
    ('North America', 'North America'),
    ('Asia Pacific', 'Asia Pacific'),
    ('Europe', 'Europe'),
    ('South America', 'South America'),
    ('Africa', 'Africa'),
    ('Australia', 'Australia'),
    ('Middle East', 'Middle East'),
    ('Global', 'Global')
)

TRAFFIC = (('', 'Not Disclosed'),
           ('0-20 Mbps', '0-20 Mbps'),
           ('20-100Mbps', '20-100Mbps'),
           ('100-1000Mbps', '100-1000Mbps'),
           ('1-5Gbps', '1-5Gbps'),
           ('5-10Gbps', '5-10Gbps'),
           ('10-20Gbps', '10-20Gbps'),
           ('20-50 Gbps', '20-50 Gbps'),
           ('50-100 Gbps', '50-100 Gbps'),
           ('100+ Gbps', '100+ Gbps'),
           ('100-200 Gbps', '100-200 Gbps'),
           ('200-300 Gbps', '200-300 Gbps'),
           ('300-500 Gbps', '300-500 Gbps'),
           ('500-1000 Gbps', '500-1000 Gbps'),
           ('1 Tbps+', '1 Tbps+'),
           ('10 Tbps+', '10 Tbps+'))

NET_TYPES = (
    ('', 'Not Disclosed'),
    ('Not Disclosed', 'Not Disclosed'),
    ('NSP', 'NSP'),
    ('Content', 'Content'),
    ('Cable/DSL/ISP', 'Cable/DSL/ISP'),
    ('Enterprise', 'Enterprise'),
    ('Educational/Research', 'Educational/Research'),
    ('Non-Profit', 'Non-Profit'),
    ('Route Server', 'Route Server')
)

VISIBILITY = (
    ('Private', 'Private'),
    #                   ('Peers', 'Peers'),
    ('Users', 'Users'),
    ('Public', 'Public'),
)




# import to namespace
from django_peeringdb.models.abstract import * # noqa

from django_peeringdb import settings as __settings

if not __settings.ABSTRACT_ONLY:
    # import to namespace
    from django_peeringdb.models.concrete import * # noqa

del __settings




import datetime
from django.core.exceptions import ValidationError, ObjectDoesNotExist, FieldDoesNotExist
from django.db import models
from django_peeringdb import settings
from decimal import Decimal
import logging


def sync_obj(cls, row):
    try:
        obj = cls.objects.get(pk=row['id'])

    except cls.DoesNotExist:
        obj = cls()
        #obj = cls(**row)

    for k, v in row.items():
        try:
            field = obj._meta.get_field(k)
        except FieldDoesNotExist, inst:
            field = None
        if field and isinstance(field, models.DecimalField) and isinstance(v, float):
            setattr(obj, k, Decimal("{:.{prec}f}".format(v, prec=field.decimal_places)))
        else:
            setattr(obj, k, v)

    try:
        obj.full_clean()

    except ValidationError as e:
        log = logging.getLogger('peeringdb.sync')
        log.debug("{} : errors: {}".format(e, e.message_dict))
        for k, v in e.message_dict.items():
            field = cls._meta.get_field(k)
            try:
                log.debug("{}: {}, dict: {}".format(k, getattr(obj, k), field.__dict__))
            except ObjectDoesNotExist:
                log.debug("{}: Missing Object, dict: {}".format(k, field.__dict__))
        raise e


    for field in cls._meta.get_fields():
        ftyp = cls._meta.get_field(field.name)
        value = getattr(obj, field.name, None)
        if settings.SYNC_STRIP_TZ and isinstance(value, datetime.datetime):
            setattr(obj, field.name, value.replace(tzinfo=None))
        else:
            if hasattr(ftyp, "related_name") and ftyp.multiple:
                continue
            else:
                setattr(obj, field.name, value)

    obj.save()
    return

    # times should always be UTC, so take off timezone since some dbs
    # don't support it - FIXME make option
    for field in cls._meta.get_fields():
        #if isinstance(field, models.ForeignKey):
        if isinstance(field, models.ForeignKey):
            key = field.name + "_id"
            value = getattr(obj, key, 0)
        else:
            key = field.name
            value = getattr(obj, field.name, None)

        if isinstance(value, datetime.datetime):
            setattr(obj, field.name, value.replace(tzinfo=None))
        else:
            setattr(obj, field.name, value)

    return




from django.conf import settings


TABLE_PREFIX = getattr(settings, 'PEERINGDB_TABLE_PREFIX', 'peeringdb_')
ABSTRACT_ONLY = getattr(settings, 'PEERINGDB_ABSTRACT_ONLY', False)

SYNC_URL = getattr(settings, 'PEERINGDB_SYNC_URL', 'https://www.peeringdb.com/api')
SYNC_USERNAME = getattr(settings, 'PEERINGDB_SYNC_USERNAME', '')
SYNC_PASSWORD = getattr(settings, 'PEERINGDB_SYNC_PASSWORD', '')

SYNC_STRIP_TZ = getattr(settings, 'PEERINGDB_SYNC_STRIP_TZ', False)
SYNC_ONLY = getattr(settings, 'PEERINGDB_SYNC_ONLY', [])

if isinstance(SYNC_ONLY, basestring):
    raise ValueError("SYNC_ONLY should be iterable (list, tuple, etc)")






import datetime
import pytest

from django.utils.timezone import make_naive
from django.core.management import (
    call_command,
    find_commands,
    load_command_class,
)
from django.test import TestCase
import django_peeringdb.models


class CoreTests(TestCase):
    """ test core functionality """

    def setUp(self):
        self.cmd = load_command_class('django_peeringdb', "pdb_sync")

    def test_get_class_list(self):
        models = self.cmd.get_class_list()
        assert len(models)
        assert django_peeringdb.models.all_models == models

        models = self.cmd.get_class_list(['network'])
        assert 1 == len(models)
        assert models[0] == django_peeringdb.models.Network

    def test_get_tag_dict(self):
        tag_dict = django_peeringdb.models.tag_dict
        assert len(tag_dict)
        for k,v in tag_dict.items():
            assert k == v._handleref.tag




"""
sync peeringdb tables
"""
from __future__ import print_function

import calendar
import logging
import re
from optparse import make_option
from twentyc.rpc import RestClient

import django.core.exceptions
from django.core.management.base import BaseCommand
from django_peeringdb import settings, sync

import django_peeringdb.models
from django_peeringdb.models.concrete import (
  Organization,
  Network,
  InternetExchange,
  Facility,
  NetworkContact,
  NetworkIXLan,
  NetworkFacility,
  InternetExchangeFacility
)

from django.apps import apps


def get_model(name):
    return apps.get_model('django_peeringdb', name)


class Command(BaseCommand):
    help = "synchronize local tables to PeeringDB"

    option_list = getattr(BaseCommand, 'option_list', ()) + (
        make_option('-n', '--dry-run',
            action='store_true',
            default=False,
            help='enable extra debug output'),
        make_option('--debug',
            action='store_true',
            default=False,
            help='enable extra debug output'),
        make_option('--only',
            action='store',
            default=False,
            help='only process this table'),
        make_option('--id',
            action='store',
            default=0,
            help='only process this id'),
        make_option('--limit',
            type=int,
            default=0,
            help="limit objects retrieved, retrieve all objects if 0 (default)"),
        )

    def handle(self, *args, **options):
        self.log = logging.getLogger('peeringdb.sync')

        kwargs = {}
        if settings.SYNC_USERNAME:
            kwargs['user'] = settings.SYNC_USERNAME
            kwargs['password'] = settings.SYNC_PASSWORD

        self.log.debug("syncing from %s", settings.SYNC_URL)
        self.connect(settings.SYNC_URL, **kwargs)

        # get models if limited by config
        only = options.get('only', settings.SYNC_ONLY)
        self.log.debug("only tables %s", only)

        pk = options.get('id', 0)

        tables = self.get_class_list(only)
        limit = options.get("limit", 0)

        # disable auto now
        for model in tables:
            for field in model._meta.fields:
                if field.name == "created":
                    field.auto_now_add = False
                if field.name == "updated":
                    field.auto_now = False

        self.sync(tables, pk, limit=limit)

    def connect(self, url, **kwargs):
        self.rpc = RestClient(url, **kwargs)

    def sync(self, tables, pk=0, **kwargs):
        for cls in tables:
            self.update_db(cls, self.get_objs(cls, pk=pk, **kwargs))

    def get_class_list(self, only=None):
        tables = []
        if only:
            for name in only:
                tables.append(get_model(name))
        else:
            tables = django_peeringdb.models.all_models
        return tables

    def get_since(self, cls):
        upd = cls.handleref.last_change()
        if upd:
            return int(calendar.timegm(upd.timetuple()))
        return 0

    def get_data(self, cls, since):
        return self.rpc.all(cls._handleref.tag, since=since)

    def get_objs(self, cls, **kwargs):
        pk = int(kwargs.pop('pk', 0))
        if pk:
            self.log.debug("getting single id=%d", pk)
            data = self.rpc.all(cls._handleref.tag, id=pk, **kwargs)
            print("%s==%d %d changed" % (cls._handleref.tag, pk, len(data)))

        else:
            since = self.get_since(cls)
            data = self.rpc.all(cls._handleref.tag, since=since, **kwargs)
            print("%s last update %s %d changed" % (cls._handleref.tag, str(since), len(data)))

        return data

    def cls_from_tag(self, tag):
        tables = self.get_class_list()
        for cls in tables:
            if cls._handleref.tag == tag:
                return cls
        raise Exception("Unknown reftag: %s" % tag)

    def _sync(self, cls, row):
        """
        Try to sync an object to the local database, in case of failure
        where a referenced object is not found, attempt to fetch said 
        object from the REST api
        """
        try:
            sync.sync_obj(cls, row)

        except django.core.exceptions.ObjectDoesNotExist as e:
            # thrown by subquery on single row
            print(e)
            raise

        except django.core.exceptions.ValidationError as inst:
            # There were validation errors
            for field, errlst in inst.error_dict.items():
                # check if it was a relationship that doesnt exist locally
                m = re.match(".+ with id (\d+) does not exist.+", str(errlst))
                if m:
                    print("%s.%s not found locally, trying to fetch object... " % (field, m.group(1)))
                    # fetch missing object
                    r = self.rpc.get(field, int(m.group(1)), depth=0)

                    # sync missing object
                    self._sync(self.cls_from_tag(field), r[0])
                else:
                    raise
           
            # try to sync initial object once more
            sync.sync_obj(cls, row)

    def update_db(self, cls, data):
        print("data to be processed", len(data))
        if not data:
            return

        for row in data:
            self._sync(cls, row)





from setuptools import find_packages, setup


version = open('facsimile/VERSION').read().strip()
requirements = open('facsimile/requirements.txt').read().split("\n")
requirements.append(open('facsimile/requirements-django.txt').read().split("\n"))
test_requirements = open('facsimile/requirements-test.txt').read().split("\n")


setup(
    name='django-peeringdb',
    version=version,
    author='PeeringDB',
    author_email='support@peeringdb.com',
    description='PeeringDB models and local synchronization for Django',
    long_description='',
    license='LICENSE.txt',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Internet',
    ],
    packages = find_packages(),
    include_package_data=True,

    url='https://github.com/peeringdb/django-peeringdb',
    download_url='https://github.com/peeringdb/django-peeringdb/%s' % version,

    install_requires=requirements,
    test_requires=test_requirements,

    zip_safe=True
)




from django.core.validators import URLValidator
from django.db import models
from django_countries.fields import CountryField
from django_handleref.models import HandleRefModel

from django_inet.models import (
    ASNField,
    IPAddressField,
    IPPrefixField,
    MacAddressField,
)
from django_peeringdb import (
    const,
    settings,
)


class URLField(models.URLField):
    default_validators = [URLValidator(schemes=["http", "https", "ftp", "ftps", "telnet"])]

    def __init__(self, *args, **kwargs):
        kwargs["max_length"] = 255
        super(URLField, self).__init__(*args, **kwargs)


class AddressModel(models.Model):
    """
    Postal Address
    """
    address1 = models.CharField(max_length=255, blank=True)
    address2 = models.CharField(max_length=255, blank=True)
    city = models.CharField(max_length=255, blank=True)
    state = models.CharField(max_length=255, blank=True)
    zipcode = models.CharField(max_length=48, blank=True)
    country = CountryField(blank=True)

    latitude = models.DecimalField(max_digits=9, decimal_places=6, blank=True, null=True, help_text="Latitude")
    longitude = models.DecimalField(max_digits=9, decimal_places=6, blank=True, null=True, help_text="Longitude")

    class Meta:
        abstract = True

    def __unicode__(self):
        return self.address1


class OrganizationBase(HandleRefModel, AddressModel):
    name = models.CharField(max_length=255, unique=True)
    website = URLField(blank=True)
    notes = models.TextField(blank=True)

    class Meta:
        abstract = True
        db_table = '%sorganization' % settings.TABLE_PREFIX
        verbose_name_plural = "Organizations"

    class HandleRef:
        tag = 'org'
        delete_cascade = ["net_set", "fac_set", "ix_set"]

    def __unicode__(self):
        return self.name


class FacilityBase(HandleRefModel, AddressModel):
    name = models.CharField(max_length=255, unique=True)
    website = URLField(blank=True)

    clli = models.CharField(max_length=18, blank=True)
    rencode = models.CharField(max_length=18, blank=True)
    npanxx = models.CharField(max_length=21, blank=True)

    notes = models.TextField(blank=True)

    class Meta:
        abstract = True
        db_table = '%sfacility' % settings.TABLE_PREFIX
        verbose_name_plural = "Facilities"

    class HandleRef:
        tag = 'fac'
        delete_cascade = ["ixfac_set", "netfac_set"]

    def __unicode__(self):
        return self.name


class ContactBase(HandleRefModel):
    role = models.CharField(max_length=27, choices=const.POC_ROLES)
    visible = models.CharField(
        max_length=64, choices=const.VISIBILITY, default='Public')
    name = models.CharField(max_length=254, blank=True)
    phone = models.CharField(max_length=100, blank=True)
    email = models.EmailField(max_length=254, blank=True)
    url = URLField(blank=True)

    class Meta:
        abstract = True
        db_table = '%snetwork_contact' % settings.TABLE_PREFIX

    class HandleRef:
        tag = 'poc'


class NetworkBase(HandleRefModel):
    asn = ASNField(unique=True)
    name = models.CharField(max_length=255, unique=True)
    aka = models.CharField(max_length=255, blank=True)
    irr_as_set = models.CharField(max_length=255, blank=True)

    website = URLField(blank=True)
    looking_glass = URLField(blank=True)
    route_server = URLField(blank=True)

    notes = models.TextField(blank=True)
    notes_private = models.TextField(blank=True)

    info_traffic = models.CharField(
        max_length=39, blank=True, choices=const.TRAFFIC)
    info_ratio = models.CharField(max_length=45, blank=True, choices=const.RATIOS,
                                  default='Not Disclosed')
    info_scope = models.CharField(max_length=39, blank=True, choices=const.SCOPES,
                                  default='Not Disclosed')
    info_type = models.CharField(max_length=60, blank=True, choices=const.NET_TYPES,
                                 default='Not Disclosed')
    info_prefixes4 = models.PositiveIntegerField(null=True, blank=True)
    info_prefixes6 = models.PositiveIntegerField(null=True, blank=True)
    info_unicast = models.BooleanField(default=False)
    info_multicast = models.BooleanField(default=False)
    info_ipv6 = models.BooleanField(default=False)

    policy_url = URLField(blank=True)
    policy_general = models.CharField(max_length=72, blank=True,
                                      choices=const.POLICY_GENERAL)
    policy_locations = models.CharField(max_length=72, blank=True,
                                        choices=const.POLICY_LOCATIONS)
    policy_ratio = models.BooleanField(default=False)
    policy_contracts = models.CharField(max_length=36, blank=True,
                                        choices=const.POLICY_CONTRACTS)

    class Meta:
        abstract = True
        db_table = '%snetwork' % settings.TABLE_PREFIX;
        verbose_name_plural = "Networks"

    class HandleRef:
        tag = 'net'
        delete_cascade = ["poc_set", "netfac_set", "netixlan_set"]

    def __unicode__(self):
        return self.name


class InternetExchangeBase(HandleRefModel):
    name = models.CharField(max_length=64, unique=True)
    name_long = models.CharField(max_length=254, blank=True)

    city = models.CharField(max_length=192)
    country = CountryField()

    notes = models.TextField(blank=True)

    region_continent = models.CharField(
        max_length=255, choices=const.REGIONS)
    media = models.CharField(max_length=128, choices=const.MEDIA)
    proto_unicast = models.BooleanField(default=False)
    proto_multicast = models.BooleanField(default=False)
    proto_ipv6 = models.BooleanField(default=False)

    website = URLField(blank=True)
    url_stats = URLField(blank=True)

    tech_email = models.EmailField(max_length=254, blank=True)
    tech_phone = models.CharField(max_length=192, blank=True)
    policy_email = models.EmailField(max_length=254, blank=True)
    policy_phone = models.CharField(max_length=192, blank=True)

    class Meta:
        abstract = True
        db_table = '%six' % settings.TABLE_PREFIX

    class HandleRef:
        tag = 'ix'
        delete_cascade = ["ixfac_set", "ixlan_set"]

    def __unicode__(self):
        return self.name


class InternetExchangeFacilityBase(HandleRefModel):

    class Meta:
        abstract = True
        db_table = '%six_facility' % settings.TABLE_PREFIX
        verbose_name_plural = "internet exchange facilities"

    class HandleRef:
        tag = 'ixfac'


class IXLanBase(HandleRefModel):
    name = models.CharField(max_length=255, blank=True)
    descr = models.TextField(blank=True)
    mtu = models.PositiveIntegerField(null=True, blank=True)
    vlan = models.PositiveIntegerField(null=True, blank=True)
    dot1q_support = models.BooleanField(default=False)
    rs_asn = ASNField(null=True, blank=True, default=0)
    arp_sponge = MacAddressField(null=True, unique=True, blank=True)

    class Meta:
        abstract = True
        db_table = '%sixlan' % settings.TABLE_PREFIX

    class HandleRef:
        tag = 'ixlan'
        delete_cascade = ["ixpfx_set", "netixlan_set"]


class IXLanPrefixBase(HandleRefModel):
    notes = models.CharField(max_length=255, blank=True)
    protocol = models.CharField(max_length=64, choices=const.PROTOCOLS)
    prefix = IPPrefixField(unique=True)

    class Meta:
        abstract = True
        db_table = '%sixlan_prefix' % settings.TABLE_PREFIX

    class HandleRef:
        tag = 'ixpfx'


class NetworkFacilityBase(HandleRefModel):
    local_asn = ASNField(null=True, blank=True)
    avail_sonet = models.BooleanField(default=False)
    avail_ethernet = models.BooleanField(default=False)
    avail_atm = models.BooleanField(default=False)

    class Meta:
        abstract = True
        db_table = '%snetwork_facility' % settings.TABLE_PREFIX
        verbose_name_plural = "Network Facilities"

    class HandleRef:
        tag = 'netfac'


class NetworkIXLanBase(HandleRefModel):
    asn = ASNField()
    ipaddr4 = IPAddressField(version=4, blank=True, null=True)
    ipaddr6 = IPAddressField(version=6, blank=True, null=True)
    is_rs_peer = models.BooleanField(default=False)
    notes = models.CharField(max_length=255, blank=True)
    speed = models.PositiveIntegerField()

    class Meta:
        abstract = True
        db_table = '%snetwork_ixlan' % settings.TABLE_PREFIX

    class HandleRef:
        tag = "netixlan"






from django.db import models
from django_peeringdb import settings
from django_peeringdb.models import (
    OrganizationBase,
    FacilityBase,
    NetworkBase,
    InternetExchangeBase,
    InternetExchangeFacilityBase,
    IXLanBase,
    IXLanPrefixBase,
    ContactBase,
    NetworkFacilityBase,
    NetworkIXLanBase,
)


all_models = []
tag_dict = {}

def expose_model(cls):
    all_models.append(cls)
    tag_dict[cls._handleref.tag] = cls
    return cls

class URLField(models.URLField):
    """
    local defaults for URLField
    """
    def __init__(self, *args, **kwargs):
        kwargs['max_length'] = 255
        super(URLField, self).__init__(*args, **kwargs)

@expose_model
class Organization(OrganizationBase):
    pass

@expose_model
class Facility(FacilityBase):
    org = models.ForeignKey(Organization, related_name="fac_set")

@expose_model
class Network(NetworkBase):
    org = models.ForeignKey(Organization, related_name="net_set")


@expose_model
class InternetExchange(InternetExchangeBase):
    org = models.ForeignKey(Organization, related_name="ix_set")

    @property
    def fac_set(self):
        return [ixfac.fac for ixfac in self.ixfac_set]

@expose_model
class InternetExchangeFacility(InternetExchangeFacilityBase):
    ix = models.ForeignKey(InternetExchange, related_name="ixfac_set")
    fac = models.ForeignKey(Facility, default=0, related_name="ixfac_set")

    class Meta:
        unique_together = ('ix', 'fac')
        db_table = '%six_facility' % settings.TABLE_PREFIX

@expose_model
class IXLan(IXLanBase):
    ix = models.ForeignKey(InternetExchange, default=0, related_name="ixlan_set")

@expose_model
class IXLanPrefix(IXLanPrefixBase):
    ixlan = models.ForeignKey(IXLan, default=0, related_name="ixpfx_set")

@expose_model
class NetworkContact(ContactBase):
    net = models.ForeignKey(Network, default=0, related_name="poc_set")

@expose_model
class NetworkFacility(NetworkFacilityBase):
    net = models.ForeignKey(Network, default=0, related_name="netfac_set")
    fac = models.ForeignKey(Facility, default=0, related_name="netfac_set")

    class Meta:
        unique_together = ('net', 'fac', 'local_asn')
        db_table = '%snetwork_facility' % settings.TABLE_PREFIX

@expose_model
class NetworkIXLan(NetworkIXLanBase):
    net = models.ForeignKey(Network, default=0, related_name="netixlan_set")
    ixlan = models.ForeignKey(IXLan, default=0, related_name="netixlan_set")

__all__ = [m.__name__ for m in all_models]
# other required vars
__all__.append('all_models')
__all__.append('tag_dict')





from django.core.exceptions import ValidationError
from django.test import TestCase
import ipaddress
import pytest

from django_peeringdb.models import (
    URLField,
)
from models import FieldModel


class FieldTests(TestCase):
    """ test model functionality """

    def test_init(self):
        new = URLField()


    def test_url(self):
        model = FieldModel()

        model.url = 'telnet://example.com'
        model.full_clean()

        model.url = 'http://example.com'
        model.full_clean()

        model.url = 'https://example.com'
        model.full_clean()

        model.url = 'ftp://example.com'
        model.full_clean()

        model.url = 'ftps://example.com'
        model.full_clean()

        with pytest.raises(ValidationError):
            model.url = 'invalid'
            model.full_clean()






from django.db import models
from django_peeringdb.models import URLField


class FieldModel(models.Model):
    url = URLField(null=True, blank=True)

    class Meta:
        app_label = 'django_peeringdb.tests'
